import { LRUCache } from "./lru";
import { ConnectFunction, InEnvelope, OutEnvelope } from "./pingingwebsocket";
import { ReconnectingWebSocket } from "./reconnectingwebsocket";
import { MutationSpec } from "./spec";
export declare const ErrorNotConnected = "not connected";
export declare const ErrorMutationTimeout = "mutation timed out";
export declare const ErrorConnectionClosed = "connection closed";
export interface GraphQLData<Result> {
    data: GraphQLResult<Result>;
}
export declare type GraphQLResult<Result> = {
    state: "error";
    error: GraphQLError;
    value: never;
} | {
    state: "loading";
    value?: Result;
    error: never;
} | {
    state: "subscribed";
    value: Result;
    error: never;
} | {
    state: "cached";
    value: Result;
    error: never;
};
export declare class GraphQLError extends Error {
    constructor(message: string);
}
export declare type SubscriptionState = "cached" | "loading" | "subscribed" | "error";
interface GraphqlQuery<InputVariables> {
    query: string;
    variables: InputVariables;
}
export interface Subscription<Result, InputVariables> extends GraphqlQuery<InputVariables> {
    state: SubscriptionState;
    retryDelay: number;
    retryHandle: any;
    observer: (data: GraphQLResult<Result>) => void;
    value: Result | undefined;
    error?: GraphQLError;
}
export interface Mutation<InputVariables> extends GraphqlQuery<InputVariables> {
    resolve: any;
    reject: any;
    timeout: number;
}
export declare type Observer<Result> = (data: Result) => void;
declare class Connection {
    nextId: number;
    subscriptions: Map<string, Subscription<object, object>>;
    mutations: Map<string, Mutation<object | undefined>>;
    past: LRUCache<{}, {}>;
    initialRetryDelay: number;
    maxRetryDelay: number;
    mutationTimeoutMs: number;
    socket: ReconnectingWebSocket;
    constructor(connectFunction: ConnectFunction);
    makeId(): string;
    subscribe<QueryResult extends object, QueryInputVariables extends object>({ query, variables, observer, }: {
        query: string;
        variables: QueryInputVariables;
        observer: Observer<GraphQLResult<QueryResult>>;
    }): {
        close: () => void;
        data: () => GraphQLResult<QueryResult>;
    };
    mutate<MutationInputVariables extends object | undefined, MutationOutput extends object>({ query, variables, }: MutationInputVariables extends undefined ? {
        query: MutationSpec<MutationOutput, MutationInputVariables>;
        variables?: undefined;
    } : {
        query: MutationSpec<MutationOutput, MutationInputVariables>;
        variables: MutationInputVariables;
    }): Promise<MutationOutput>;
    mutate<MutationInputVariables extends object | undefined, MutationOutput>({ query, variables, }: MutationInputVariables extends undefined ? {
        query: string;
        variables?: undefined;
    } : {
        query: string;
        variables: MutationInputVariables;
    }): Promise<MutationOutput>;
    notify(subscription: Subscription<object, object>): void;
    retry(id: string): void;
    open(): void;
    close(): void;
    send(message: OutEnvelope): void;
    handleOpen(): void;
    handleClose(): void;
    handleMessage(envelope: InEnvelope): void;
}
export { Connection };
