var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import React from "react";
import { isEqual } from "lodash";
import { Consumer } from "./context";
export function Query(props) {
    var _a;
    let passedProps;
    let query;
    if (typeof props.query === "string") {
        (_a = props, { query } = _a, passedProps = __rest(_a, ["query"]));
    }
    else {
        const { query: querySpec } = props, shouldPass = __rest(props, ["query"]);
        passedProps = shouldPass;
        query = querySpec.query;
    }
    return (React.createElement(Consumer, null, connection => (React.createElement(GraphQLRenderer, Object.assign({ connection: connection, query: query }, passedProps)))));
}
class GraphQLRenderer extends React.PureComponent {
    componentWillMount() {
        const { query, variables } = this.props;
        this.subscribe({ query, variables });
    }
    componentWillReceiveProps(nextProps) {
        const { query, variables } = nextProps;
        if (isEqual(query, this.state.query) &&
            isEqual(variables, this.state.variables)) {
            return;
        }
        this.subscribe({ query, variables });
    }
    componentWillUnmount() {
        this.unsubscribe();
    }
    render() {
        const { query, variables } = this.props;
        // If the current state is valid (subscribed/cached), we can render the data.
        const hasValidValue = this.state.state === "subscribed" || this.state.state === "cached";
        // If we are loading a new query and the previous query is the same (but with different query variables),
        // show the last value if it exists.
        const hasPreviousValue = this.state.state === "loading" && isEqual(query, this.state.query);
        // If we are rendering an error, show the last value if it exists are the query
        // and its variables are exactly the same. (i.e. a refresh of the data failed).
        const hasValueFromBeforeError = this.state.state === "error" &&
            isEqual(query, this.state.query) &&
            isEqual(variables, this.state.variables);
        const shouldRenderValue = hasValidValue || hasPreviousValue || hasValueFromBeforeError;
        const data = {
            value: shouldRenderValue ? this.state.value : undefined,
            state: this.state.state,
            error: this.state.error,
        };
        return this.props.children(data);
    }
    onData(data, query, variables) {
        let partialState = null;
        if (data.state === "subscribed" || data.state === "cached") {
            partialState = {
                value: data.value,
                query,
                variables,
            };
        }
        this.setState(Object.assign({ state: data.state, error: data.error }, partialState));
    }
    subscribe({ query, variables, }) {
        this.unsubscribe();
        this.subscription = this.props.connection.subscribe({
            query,
            variables,
            observer: data => this.onData(data, query, variables),
        });
        this.onData(this.subscription.data(), query, variables);
    }
    unsubscribe() {
        if (this.subscription) {
            this.subscription.close();
            this.subscription = undefined;
        }
    }
}
