import { BasicWebSocket } from "./basicwebsocket";
import { PingingWebSocket, } from "./pingingwebsocket";
export const reconnectDelayMs = 10000;
export class ReconnectingWebSocket {
    constructor(connectFunction, listener) {
        this.socket = undefined;
        this.reconnectHandle = undefined;
        this.hadSuccess = false;
        this.reconnect = () => {
            this.destroySocket();
            this.hadSuccess = false;
            this.socket = new PingingWebSocket(this.connectFunction, {
                onOpen: () => {
                    this.listener.onOpen();
                },
                onMessage: (envelope) => {
                    this.hadSuccess = true;
                    this.listener.onMessage(envelope);
                },
                onClose: (reason) => {
                    this.destroySocket();
                    // attempt to reconnect after a reconnectDelay
                    const waitMs = this.hadSuccess ? 0 : reconnectDelayMs;
                    this.reconnectHandle = setTimeout(this.reconnect, waitMs);
                    this.listener.onClose(reason);
                },
            });
            return this.socket.connect();
        };
        this.connectFunction = connectFunction;
        this.listener = listener;
    }
    get state() {
        if (this.socket) {
            switch (this.socket.readyState) {
                case BasicWebSocket.CONNECTING:
                    return "connecting";
                case BasicWebSocket.OPEN:
                    return "connected";
                default:
                    return "waiting to reconnect";
            }
        }
        if (this.reconnectHandle) {
            return "waiting to reconnect";
        }
        else {
            return "closed";
        }
    }
    get ravenTags() {
        return {
            readyState: this.socket && this.socket.readyState,
            state: this.state,
        };
    }
    send(message) {
        if (this.socket) {
            this.socket.send(message);
        }
    }
    close() {
        this.destroySocket();
    }
    destroySocket() {
        if (this.socket) {
            this.socket.close();
            this.socket = undefined;
        }
        if (this.reconnectHandle) {
            clearTimeout(this.reconnectHandle);
            this.reconnectHandle = undefined;
        }
    }
}
